
//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <SI_EFM8UB1_Register_Enums.h>                  // SFR declarations
#include "InitDevice.h"
#include "VCPXpress.h"
#include "descriptor.h"
#include "SoundSynthesizer_Defs.h"

bit REPEATED_NOTE;
unsigned char CurrentNoteIndex;

//-----------------------------------------------------------------------------
// main() Routine
// ----------------------------------------------------------------------------
int main (void)
{
	// Call hardware initialization routine
	enter_DefaultMode_from_RESET();

	// VCPXpress Initialization
	USB_Init(&InitStruct);

	// Enable VCPXpress API interrupts
	API_Callback_Enable(myAPICallback);

	/*Initially we stop the PCA counter, so no sound will be generated
	until the "play" command is sent through the USB interface*/
	SFRPAGE = PCA0_PAGE;
	PCA0CN0_CR = STOP;

	// Enable global interrupts
	IE_EA = 1;

	while(1)
   {
		for(CurrentNoteIndex = 0; CurrentNoteIndex < NumberofNotes; CurrentNoteIndex++)
		{
			REPEATED_NOTE = FALSE;
			if(CurrentNoteIndex < (NumberofNotes - 1))
			{
				if(NotesSequence[CurrentNoteIndex] == NotesSequence[CurrentNoteIndex + 1])
					REPEATED_NOTE = TRUE;
			}

			switch(NotesSequence[CurrentNoteIndex])
			{
				case 'c':
					Current_SoundSignal_Increment = SOUND_C5_INCREMENT;
					Current_FilterClock_Increment = FILTCLK_C5_INCREMENT;
					break;

				case 'd':
					Current_SoundSignal_Increment = SOUND_D5_INCREMENT;
					Current_FilterClock_Increment = FILTCLK_D5_INCREMENT;
					break;

				case 'e':
					Current_SoundSignal_Increment = SOUND_E5_INCREMENT;
					Current_FilterClock_Increment = FILTCLK_E5_INCREMENT;
					break;

				case 'f':
					Current_SoundSignal_Increment = SOUND_F5_INCREMENT;
					Current_FilterClock_Increment = FILTCLK_F5_INCREMENT;
					break;

				case 'g':
					Current_SoundSignal_Increment = SOUND_G5_INCREMENT;
					Current_FilterClock_Increment = FILTCLK_G5_INCREMENT;
					break;

				case 'a':
					Current_SoundSignal_Increment = SOUND_A5_INCREMENT;
					Current_FilterClock_Increment = FILTCLK_A5_INCREMENT;
					break;

				case 'b':
					Current_SoundSignal_Increment = SOUND_B5_INCREMENT;
					Current_FilterClock_Increment = FILTCLK_B5_INCREMENT;
					break;

				case 'C':
					Current_SoundSignal_Increment = SOUND_C6_INCREMENT;
					Current_FilterClock_Increment = FILTCLK_C6_INCREMENT;
					break;

				case 'D':
					Current_SoundSignal_Increment = SOUND_D6_INCREMENT;
					Current_FilterClock_Increment = FILTCLK_D6_INCREMENT;
					break;

				case 'E':
					Current_SoundSignal_Increment = SOUND_E6_INCREMENT;
					Current_FilterClock_Increment = FILTCLK_E6_INCREMENT;
					break;

				case 'r':
					SFRPAGE = PCA0_PAGE;
					PCA0CN0_CR = STOP;
					break;
			}

			//delay for the length of time corresponding to the current note
			SFRPAGE = TIMER3_PAGE; TMR3 = 0; while(TMR3 < Timer3Counts_Sequence[CurrentNoteIndex]);

			//add a short pause to provide separation between repeated notes
			if(REPEATED_NOTE == TRUE)
			{
				SFRPAGE = PCA0_PAGE;
				PCA0CN0_CR = STOP;
				SFRPAGE = TIMER3_PAGE; TMR3 = 0; while(TMR3 < 200);
			}

			SFRPAGE = PCA0_PAGE;
			PCA0CN0_CR = PLAYorSTOP;
		}

		//pause for one second before repeating the melody
		SFRPAGE = PCA0_PAGE;
		PCA0CN0_CR = STOP;
		SFRPAGE = TIMER3_PAGE; TMR3 = 0; while(TMR3 < 10000);

		SFRPAGE = PCA0_PAGE;
		PCA0CN0_CR = PLAYorSTOP;
   }                             
}


