/*
 * USBInterface.c
 *      Author: RK
 */

#include <SI_EFM8UB1_Register_Enums.h>                  // SFR declarations
#include "VCPXpress.h"
#include "descriptor.h"
#include "SoundSynthesizer_Defs.h"

uint16_t xdata USBBytesReceived = 0;                   // Holds size of received packet
uint16_t xdata USBBytesTransmitted = 0;			//Holds number of bytes actually transmitted
uint8_t xdata USBRxPacket[USB_PACKET_SIZE];     // Packet received from host

bit USB_PACKET_RECEIVED = FALSE;
unsigned char Response_String[] = "EFM8 Received: ";
bit HEADER_SENT = FALSE;

unsigned int WholeNoteCounts = 20000;
unsigned int HalfNoteCounts = 10000;
unsigned int QuarterNoteCounts = 5000;
unsigned int EighthNoteCounts = 2500;

unsigned int xdata Timer3Counts_Sequence[32];
unsigned char xdata NotesSequence[32];
unsigned char NumberofNotes;

bit PLAYorSTOP = STOP;
bit SEND_LAST_BYTE = FALSE;




/**************************************************************************//**
 * VCPXpress callback
 * This function is called by VCPXpress.
 *****************************************************************************/
VCPXpress_API_CALLBACK(myAPICallback)
{
   uint32_t API_InterruptCode;

   //get the code that indicates the reason for the interrupt
   API_InterruptCode = Get_Callback_Source();

   //if the USB connection was just opened
   if (API_InterruptCode & DEVICE_OPEN)
   {
	   //start the first USB read procedure
	   Block_Read(USBRxPacket, USB_PACKET_SIZE, &USBBytesReceived);
	   /*we will process the received bytes when we get
	   a callback with an RX_COMPLETE interrupt code*/
   }

   if (API_InterruptCode & RX_COMPLETE)   // USB read complete
   {
	   ProcessUSBRxPacket();

	   //continue with the next USB read procedure
	   Block_Read(USBRxPacket, USB_PACKET_SIZE, &USBBytesReceived);
   }

   if (API_InterruptCode & TX_COMPLETE)   // USB transmission complete
   {
	   //if we just sent the "EFM8 Received:" header, now echo the received data
	   if(HEADER_SENT)
	   {
		   /*This block of code contains the following hack: for some reason the Scilab serial port library
		    * does not properly read the echoed data when a full 64-byte packet is transmitted. The data
		    * does appear in the receive buffer, but not until after the next command is sent from Scilab.
		    * Consequently, the echoed data is not displayed by Scilab immediately after a 64-byte
		    * command is sent. We avoid this strange behavior by checking to see if the previous received
		    * packet was 64 bytes in length. If so, we split up the echo into two transmissions: the first
		    * transmission sends the first 63 bytes, and the second sends the 64th byte.*/
		   if(USBBytesReceived == USB_PACKET_SIZE)
		   {
			   if(SEND_LAST_BYTE == FALSE)
			   {
				   Block_Write(USBRxPacket, (USB_PACKET_SIZE - 1), &USBBytesTransmitted);
				   SEND_LAST_BYTE = TRUE;
			   }

			   else
			   {
				   Block_Write((USBRxPacket + (USB_PACKET_SIZE - 1)), 1, &USBBytesTransmitted);
				   SEND_LAST_BYTE = FALSE;
				   HEADER_SENT = FALSE;
			   }
		   }

		   else
		   {
			   Block_Write(USBRxPacket, USBBytesReceived, &USBBytesTransmitted);
			   HEADER_SENT = FALSE;
		   }
		}
   }
}

void ProcessUSBRxPacket()
{
	//stop playback while we are processing the packet
	SFRPAGE = PCA0_PAGE;
	PCA0CN0_CR = STOP;

	switch(USBRxPacket[0])
	{
		case 'T':
			SetTempo(USBRxPacket[1]);
			break;

		case 'P':
			PLAYorSTOP = PLAY;
			break;

		case 'S':
			PLAYorSTOP = STOP;
			break;

		case 'c':
		case 'd':
		case 'e':
		case 'f':
		case 'g':
		case 'a':
		case 'b':
		case 'C':
		case 'D':
		case 'E':
			StoreNewMelody();
			break;
	}

	/*first transmit the "EFM8 Received:" header, and then
	 the received data will be echoed after this transmission
	 is complete*/
	Block_Write(Response_String, 15, &USBBytesTransmitted);
	HEADER_SENT = TRUE;
}

void SetTempo(unsigned char TempoBPM)
{
	if(TempoBPM > 180)
		TempoBPM = 180;

	else if(TempoBPM < 60)
		TempoBPM = 60;

	QuarterNoteCounts = ((float)60/TempoBPM) * 10000;

	WholeNoteCounts = QuarterNoteCounts*4;
	HalfNoteCounts = QuarterNoteCounts*2;
	EighthNoteCounts = QuarterNoteCounts/2;
}

void StoreNewMelody()
{
	unsigned char x,y;

	y = 0;

	for(x = 0; x < USBBytesReceived; x += 2)
	{
		NotesSequence[y] = USBRxPacket[x];

		switch(USBRxPacket[x+1])
		{
			case '1':
				Timer3Counts_Sequence[y] = WholeNoteCounts;
				break;
			case '2':
				Timer3Counts_Sequence[y] = HalfNoteCounts;
				break;
			case '4':
				Timer3Counts_Sequence[y] = QuarterNoteCounts;
				break;
			case '8':
				Timer3Counts_Sequence[y] = EighthNoteCounts;
				break;
		}

		y++;
	}

	NumberofNotes = y;

	CurrentNoteIndex = NumberofNotes - 1;		//start playback at the beginning of the melody

	PLAYorSTOP = PLAY;
}


